use crate::datastore::Store;
use std::sync::Arc;

use salvo::prelude::*;
use salvo::affix;

#[handler]
async fn homepage(depot: &mut Depot) -> String {
    let ds = depot.obtain::<Arc<Store>>().unwrap();
    ds.tabled_node_list().await
}

#[handler]
async fn sign(req: &mut Request, depot: &mut Depot) -> String {
    let ds = depot.obtain::<Arc<Store>>().unwrap();
    let pubkey = match req.query::<String>("pubkey") {
        Some(k) => k,
        None => return "pubkey must be specified as GET param".to_string(),
    };

    let something = match req.query::<String>("something") {
        Some(k) => k,
        None => return "something must be specified as GET param".to_string(),
    };

    match ds.sign_message_with_key(&something, &pubkey).await {
        Ok(s) => s,
        Err(e) => e.to_string(),
    }
}

pub fn init(ds: Arc<Store>) -> std::thread::JoinHandle<()> {
    std::thread::spawn(|| {
        tokio::runtime::Runtime::new().unwrap().block_on(async {
            let acceptor = TcpListener::new("0.0.0.0:31372").bind().await;
            let router = Router::new()
                .hoop(affix::inject(ds))
                .get(homepage)
                .push(Router::with_path("sign").get(sign));
            println!("{:?}", router);
            Server::new(acceptor).serve(router).await;
        });
    })
}
